import { Correlation, FileSystem, FileSystemReadSync } from './api';
import { Directory, DirectoryContent, File, ShallowDirectory, SimpleStats } from "./model";
import { InternalEventsEmitter } from './utils';
export declare namespace CacheFileSystem {
    interface Options {
        /**
         * should the cache re-sync when an error is reported from the underlying FS
         * default : true
         */
        reSyncOnError?: boolean;
        /**
         * should sync read methods propagate to underlying FS
         * default : false
         */
        propagateSyncRead?: boolean;
    }
}
export declare class CacheFileSystem implements FileSystemReadSync, FileSystem {
    private options;
    baseUrl: string;
    fs: FileSystem;
    private cache;
    readonly events: InternalEventsEmitter;
    private pathsInCache;
    private onFsError;
    constructor(fs: FileSystem, options?: CacheFileSystem.Options);
    private isPropagateSyncRead();
    private cacheTextFile(fullPath, newContent, correlation?);
    private cacheDirTree(fullPath, newTree);
    private cacheDirChildren(fullPath, newChildren);
    private cacheStat(fullPath, newStat);
    private cacheDirectoryGone(recursive, fullPath, correlation?);
    private cacheDirectoryExists(fullPath, correlation?);
    private cacheFileGone(fullPath, correlation?);
    saveFile(fullPath: string, newContent: string, correlation?: Correlation): Promise<Correlation>;
    deleteFile(fullPath: string, correlation?: Correlation): Promise<Correlation>;
    deleteDirectory(fullPath: string, recursive?: boolean, correlation?: Correlation): Promise<Correlation>;
    ensureDirectory(fullPath: string, correlation?: Correlation): Promise<Correlation>;
    loadTextFile(fullPath: string): Promise<string>;
    loadTextFileSync(fullPath: string): string;
    loadDirectoryTree(fullPath?: string): Promise<Directory>;
    loadDirectoryTreeSync(fullPath?: string): Directory;
    loadDirectoryContentSync(fullPath?: string): DirectoryContent;
    loadDirectoryChildren(fullPath: string): Promise<(File | ShallowDirectory)[]>;
    loadDirectoryChildrenSync(fullPath: string): Array<File | ShallowDirectory>;
    stat(fullPath: string): Promise<SimpleStats>;
    statSync(fullPath: string): SimpleStats;
    dispose(): void;
    private reSyncOnError();
    private emit(type, data);
}
