"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var promise_utils_1 = require("./promise-utils");
var EventsManager = /** @class */ (function () {
    function EventsManager(options) {
        if (options === void 0) { options = {}; }
        this.options = options;
        this.events = utils_1.makeEventsEmitter();
        this.eventHandlers = new Set();
    }
    EventsManager.prototype.emit = function (e) {
        var _this = this;
        if (this.options.delay) {
            promise_utils_1.delayedPromise(this.options.delay)
                .then(function () { return _this._emit(e); });
        }
        else {
            this._emit(e);
        }
    };
    EventsManager.prototype.addEventHandler = function (handler, timeout) {
        var _this = this;
        var _handler = handler;
        if (timeout) {
            (_handler).timer = setTimeout(function () { return _this.eventHandlers.delete(_handler); }, timeout);
        }
        this.eventHandlers.add(_handler);
    };
    EventsManager.prototype.removeEventHandler = function (handler) {
        this.eventHandlers.delete(handler);
        clearTimeout(handler.timer);
    };
    EventsManager.prototype._emit = function (e) {
        this.eventHandlers.forEach(function (handler) {
            if (e && ~handler.types.indexOf(e.type) && handler.filter(e)) {
                e = handler.apply(e);
            }
        });
        if (e) {
            this.events.emit(e.type, e);
        }
    };
    return EventsManager;
}());
exports.EventsManager = EventsManager;
//# sourceMappingURL=events-manager.js.map