"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var klaw = require("klaw");
var memory_fs_1 = require("./memory-fs");
var model_1 = require("./model");
var fs_extra_1 = require("fs-extra");
var path = require("path");
exports.KLAW_SHALLOW_OPTIONS = { depthLimit: 0 };
function klawItemsToMemFs(items, baseUrl, readFile) {
    var memFs = new memory_fs_1.MemoryFileSystem();
    items.forEach(function (item) {
        var itemPath = path.relative(baseUrl, item.path).split(path.sep).join(model_1.pathSeparator);
        if (item.stats.isDirectory()) {
            memFs.ensureDirectorySync(itemPath);
        }
        else if (item.stats.isFile()) {
            memFs.saveFileSync(itemPath, readFile ? fs_extra_1.readFileSync(item.path, 'utf8') : '');
        }
        else {
            console.warn("unknown node type at " + itemPath, item);
        }
    });
    return memFs;
}
exports.klawItemsToMemFs = klawItemsToMemFs;
function klawAsPromised(rootPath, options) {
    return new Promise(function (resolve, reject) {
        var items = [];
        var walker = klaw(rootPath, options);
        walker
            .on('readable', function () {
            var item;
            while ((item = walker.read())) {
                items.push(item);
            }
        })
            .on('error', reject)
            .on('end', function () {
            resolve(items);
        });
    });
}
exports.klawAsPromised = klawAsPromised;
//# sourceMappingURL=klaw.js.map