"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs_extra_1 = require("fs-extra");
var path = require("path");
var utils_1 = require("./utils");
var klaw_1 = require("./klaw");
var klawSync = require("klaw-sync");
function statToSimpleStat(nodeStat, fullPath) {
    if (nodeStat.isDirectory()) {
        return { type: 'dir' };
    }
    else if (nodeStat.isFile()) {
        return { type: 'file' };
    }
    throw new Error("Unsupported type " + fullPath);
}
function getStatsForDeletion(baseUrl, relPath) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var fullPath, _a, e_1;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    if (!relPath) {
                        throw new Error("Can't delete root directory");
                    }
                    fullPath = path.join(baseUrl, relPath);
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 4, , 5]);
                    return [4 /*yield*/, fs_extra_1.access(fullPath)];
                case 2:
                    _b.sent();
                    _a = { fullPath: fullPath };
                    return [4 /*yield*/, fs_extra_1.stat(fullPath)];
                case 3: return [2 /*return*/, (_a.stats = _b.sent(), _a)];
                case 4:
                    e_1 = _b.sent();
                    return [2 /*return*/, null];
                case 5: return [2 /*return*/];
            }
        });
    });
}
var LocalFileSystemCrudOnly = /** @class */ (function () {
    function LocalFileSystemCrudOnly(baseUrl) {
        this.baseUrl = baseUrl;
    }
    LocalFileSystemCrudOnly.prototype.saveFile = function (relPath, newContent) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a, fullPath, name;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this.getPathAndName(relPath), fullPath = _a.fullPath, name = _a.name;
                        return [4 /*yield*/, fs_extra_1.ensureDir(fullPath)];
                    case 1:
                        _b.sent();
                        return [4 /*yield*/, fs_extra_1.writeFile(path.join(fullPath, name), newContent)];
                    case 2:
                        _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    LocalFileSystemCrudOnly.prototype.deleteFile = function (relPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var res;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, getStatsForDeletion(this.baseUrl, relPath)];
                    case 1:
                        res = _a.sent();
                        if (!res) return [3 /*break*/, 4];
                        if (!res.stats.isFile()) return [3 /*break*/, 3];
                        return [4 /*yield*/, fs_extra_1.remove(res.fullPath)];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3: throw new Error("not a file: " + relPath);
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    LocalFileSystemCrudOnly.prototype.deleteDirectory = function (relPath, recursive) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var res;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, getStatsForDeletion(this.baseUrl, relPath)];
                    case 1:
                        res = _a.sent();
                        if (!res) return [3 /*break*/, 4];
                        if (!res.stats.isDirectory()) return [3 /*break*/, 3];
                        return [4 /*yield*/, (recursive ? fs_extra_1.remove(res.fullPath) : fs_extra_1.rmdir(res.fullPath))];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3: throw new Error("not a directory: " + relPath);
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    LocalFileSystemCrudOnly.prototype.loadTextFile = function (relPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, fs_extra_1.readFile(path.join(this.baseUrl, relPath), 'utf8')];
            });
        });
    };
    LocalFileSystemCrudOnly.prototype.loadTextFileSync = function (relPath) {
        return fs_extra_1.readFileSync(path.join(this.baseUrl, relPath), 'utf8');
    };
    LocalFileSystemCrudOnly.prototype.loadDirectoryChildren = function (fullPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var rootPath, items, memFs;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        rootPath = fullPath ? path.join(this.baseUrl, fullPath) : this.baseUrl;
                        return [4 /*yield*/, klaw_1.klawAsPromised(rootPath, klaw_1.KLAW_SHALLOW_OPTIONS)];
                    case 1:
                        items = _a.sent();
                        memFs = klaw_1.klawItemsToMemFs(items, this.baseUrl, false);
                        return [2 /*return*/, memFs.loadDirectoryChildren(fullPath)];
                }
            });
        });
    };
    LocalFileSystemCrudOnly.prototype.loadDirectoryChildrenSync = function (fullPath) {
        var rootPath = fullPath ? path.join(this.baseUrl, fullPath) : this.baseUrl;
        var items = klawSync(rootPath, klaw_1.KLAW_SHALLOW_OPTIONS);
        var memFs = klaw_1.klawItemsToMemFs(items, this.baseUrl, false);
        return memFs.loadDirectoryChildrenSync(fullPath);
    };
    LocalFileSystemCrudOnly.prototype.loadDirectoryTree = function (fullPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var rootPath, items, memFs;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        rootPath = fullPath ? path.join(this.baseUrl, fullPath) : this.baseUrl;
                        return [4 /*yield*/, klaw_1.klawAsPromised(rootPath)];
                    case 1:
                        items = _a.sent();
                        memFs = klaw_1.klawItemsToMemFs(items, this.baseUrl, false);
                        return [2 /*return*/, memFs.loadDirectoryTreeSync(fullPath)];
                }
            });
        });
    };
    LocalFileSystemCrudOnly.prototype.loadDirectoryTreeSync = function (fullPath) {
        var rootPath = fullPath ? path.join(this.baseUrl, fullPath) : this.baseUrl;
        var items = klawSync(rootPath);
        var memFs = klaw_1.klawItemsToMemFs(items, this.baseUrl, false);
        return memFs.loadDirectoryTreeSync(fullPath);
    };
    LocalFileSystemCrudOnly.prototype.loadDirectoryContentSync = function (fullPath) {
        if (fullPath === void 0) { fullPath = ''; }
        var rootPath = fullPath ? path.join(this.baseUrl, fullPath) : this.baseUrl;
        var items = klawSync(rootPath);
        var memFs = klaw_1.klawItemsToMemFs(items, this.baseUrl, true);
        return memFs.loadDirectoryContentSync(fullPath);
    };
    LocalFileSystemCrudOnly.prototype.stat = function (fullPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = statToSimpleStat;
                        return [4 /*yield*/, fs_extra_1.stat(path.join(this.baseUrl, fullPath))];
                    case 1: return [2 /*return*/, _a.apply(void 0, [_b.sent(), fullPath])];
                }
            });
        });
    };
    LocalFileSystemCrudOnly.prototype.statSync = function (fullPath) {
        return statToSimpleStat(fs_extra_1.statSync(path.join(this.baseUrl, fullPath)), fullPath);
    };
    LocalFileSystemCrudOnly.prototype.ensureDirectory = function (relPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var pathArr, fullPath;
            return tslib_1.__generator(this, function (_a) {
                pathArr = utils_1.getPathNodes(relPath);
                fullPath = path.join.apply(path, [this.baseUrl].concat(pathArr));
                return [2 /*return*/, fs_extra_1.ensureDir(fullPath)];
            });
        });
    };
    LocalFileSystemCrudOnly.prototype.getPathAndName = function (relPath) {
        var pathArr = utils_1.getPathNodes(relPath);
        var name = pathArr.pop() || '';
        var fullPath = path.join.apply(path, [this.baseUrl].concat(pathArr));
        return { fullPath: fullPath, name: name };
    };
    return LocalFileSystemCrudOnly;
}());
exports.LocalFileSystemCrudOnly = LocalFileSystemCrudOnly;
//# sourceMappingURL=local-fs-crud-only.js.map