import { RetryPromiseOptions } from './promise-utils';
import { Correlation, EventEmitter, FileSystemReadSync } from './api';
import { Directory, DirectoryContent, File, ShallowDirectory, SimpleStats } from './model';
export declare namespace LocalFileSystem {
    type Options = RetryPromiseOptions & {
        correlationWindow: number;
        noiseReduceWindow: number;
    };
}
export declare class LocalFileSystem implements FileSystemReadSync {
    baseUrl: string;
    private options;
    private readonly eventsManager;
    readonly events: EventEmitter;
    private crud;
    private watcher?;
    private tempEventsTimers;
    private lastEvents;
    constructor(baseUrl: string, options?: LocalFileSystem.Options);
    private isDuplicatedEvent(ev);
    init(): Promise<LocalFileSystem>;
    dispose(): void;
    saveFile(fullPath: string, newContent: string, correlation?: string): Promise<Correlation>;
    deleteFile(fullPath: string, correlation?: string): Promise<Correlation>;
    deleteDirectory(fullPath: string, recursive?: boolean, correlation?: string): Promise<Correlation>;
    ensureDirectory(fullPath: string, correlation?: string): Promise<Correlation>;
    loadTextFile(fullPath: string): Promise<string>;
    loadDirectoryTree(fullPath?: string): Promise<Directory>;
    loadDirectoryChildren(fullPath: string): Promise<(File | ShallowDirectory)[]>;
    stat(fullPath: string): Promise<SimpleStats>;
    private registerCorrelationForPathsInDir(fullPath, correlation);
    private registerCorrelator<S>(types, correlation, filter, single);
    loadTextFileSync(fullPath: string): string;
    loadDirectoryTreeSync(fullPath?: string): Directory;
    loadDirectoryContentSync(fullPath?: string): DirectoryContent;
    loadDirectoryChildrenSync(fullPath: string): Array<File | ShallowDirectory>;
    statSync(fullPath: string): SimpleStats;
}
