import { Correlation, Events, FileSystem, FileSystemReadSync } from "./api";
import { Directory, DirectoryContent, File, ShallowDirectory, SimpleStats } from "./model";
import { InternalEventsEmitter } from "./utils";
export declare namespace MemoryFileSystem {
    interface Options {
        content?: DirectoryContent;
        model?: Directory;
    }
}
export declare class MemoryFileSystem implements FileSystemReadSync, FileSystem {
    baseUrl: string;
    static addContent(fs: MemoryFileSystem, content: DirectoryContent, path?: string): void;
    readonly events: InternalEventsEmitter;
    protected readonly root: Directory;
    constructor(baseUrl?: string, options?: MemoryFileSystem.Options);
    protected emit<S extends keyof Events>(type: S, event: Events[S]): void;
    saveFile(fullPath: string, newContent: string, correlation?: Correlation): Promise<Correlation>;
    deleteFile(fullPath: string, correlation?: Correlation): Promise<Correlation>;
    deleteDirectory(fullPath: string, recursive?: boolean, correlation?: Correlation): Promise<Correlation>;
    ensureDirectory(fullPath: string, correlation?: Correlation): Promise<Correlation>;
    loadTextFile(fullPath: string): Promise<string>;
    loadDirectoryTree(fullPath?: string): Promise<Directory>;
    loadDirectoryChildren(fullPath: string): Promise<(File | ShallowDirectory)[]>;
    stat(fullPath: string): Promise<SimpleStats>;
    saveFileSync(fullPath: string, newContent: string, correlation?: Correlation): Correlation;
    deleteFileSync(fullPath: string, correlation?: Correlation): Correlation;
    deleteDirectorySync(fullPath: string, recursive?: boolean, correlation?: Correlation): Correlation;
    ensureDirectorySync(fullPath: string, correlation?: Correlation): Correlation;
    protected findNode(fullPath: string): Directory | File;
    protected getDir(fullPath: string): Directory;
    loadTextFileSync(fullPath: string): string;
    loadDirectoryContentSync(fullPath?: string): DirectoryContent;
    loadDirectoryTreeSync(fullPath?: string): Directory;
    loadDirectoryChildrenSync(fullPath: string): (File | ShallowDirectory)[];
    statSync(fullPath: string): SimpleStats;
    private _ensureDirectorySync(fullPath, correlation);
    private recursiveEmitDeletion(node, correlation);
}
