"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
exports.pathSeparator = '/';
var ShallowDirectory = /** @class */ (function () {
    function ShallowDirectory(name, fullPath) {
        this.name = name;
        this.fullPath = fullPath;
        this.type = 'dir';
    }
    return ShallowDirectory;
}());
exports.ShallowDirectory = ShallowDirectory;
var Directory = /** @class */ (function () {
    function Directory(name, fullPath, children) {
        if (children === void 0) { children = []; }
        this.name = name;
        this.fullPath = fullPath;
        this.children = children;
        this.type = 'dir';
    }
    Directory.walk = function (directory, path) {
        var pathArr = typeof path === 'string' ? utils_1.getPathNodes(path) : path;
        var _loop_1 = function () {
            var targetName = pathArr.shift();
            if (targetName && directory.children) {
                var node = directory.children.find(function (_a) {
                    var name = _a.name;
                    return name === targetName;
                });
                if (isDir(node)) {
                    directory = node;
                }
                else {
                    return { value: null };
                }
            }
            else {
                return { value: null };
            }
        };
        while (pathArr.length) {
            var state_1 = _loop_1();
            if (typeof state_1 === "object")
                return state_1.value;
        }
        return directory;
    };
    Directory.getSubDir = function (directory, path) {
        var pathArr = typeof path === 'string' ? utils_1.getPathNodes(path) : path;
        var _loop_2 = function () {
            var targetName = pathArr.shift();
            if (targetName && directory.children) {
                var node = directory.children.find(function (_a) {
                    var name = _a.name;
                    return name === targetName;
                });
                if (isDir(node)) {
                    directory = node;
                }
                else {
                    return { value: null };
                }
            }
            else {
                return { value: null };
            }
        };
        while (pathArr.length) {
            var state_2 = _loop_2();
            if (typeof state_2 === "object")
                return state_2.value;
        }
        return directory;
    };
    Directory.clone = function (node, path) {
        var _this = this;
        if (path === void 0) { path = []; }
        var pathArr = typeof path === 'string' ? utils_1.getPathNodes(path) : path;
        if (!pathArr.length && node.name) {
            pathArr.push(node.name);
        }
        return new Directory(pathArr.length ? pathArr[pathArr.length - 1] : node.name, utils_1.normalizePathNodes(pathArr), node.children.map(function (child) {
            var childPath = pathArr.concat([child.name]);
            if (isDir(child)) {
                return _this.clone(child, childPath);
            }
            else {
                return new File(child.name, utils_1.normalizePathNodes(childPath), child.content);
            }
        }));
    };
    Directory.cloneStructure = function (node) {
        var _this = this;
        return new Directory(node.name, node.fullPath, node.children.map(function (child) { return isDir(child) ? _this.cloneStructure(child) : new File(child.name, child.fullPath); }));
    };
    Directory.fromContent = function (content, name, location) {
        var _this = this;
        if (name === void 0) { name = ''; }
        if (location === void 0) { location = ''; }
        if (location === exports.pathSeparator) {
            location = '';
        }
        else if (location.length && !utils_1.endsWith(location, exports.pathSeparator)) {
            location = location + exports.pathSeparator;
        }
        var path = location.length ? location + name : name;
        var childLocation = path.length ? path + exports.pathSeparator : name;
        return new Directory(name, path, Object.keys(content).map(function (contentPartPath) {
            var fileContent = content[contentPartPath];
            var fullPath = childLocation + contentPartPath;
            if (typeof fileContent === 'string') {
                return new File(contentPartPath, fullPath, fileContent);
            }
            else {
                return _this.fromContent(fileContent, contentPartPath, childLocation);
            }
        }));
    };
    Directory.toContent = function (directory) {
        var _this = this;
        var result = {};
        directory.children.forEach(function (child) {
            if (child.type === 'file') {
                if (child.content === undefined) {
                    throw new Error('file not loaded : ' + child.fullPath);
                }
                result[child.name] = child.content;
            }
            else if (child.type === 'dir') {
                result[child.name] = _this.toContent(child);
            }
        });
        return result;
    };
    /**
     * add one directory to another, in-place
     * @returns {Directory} subject argument
     */
    Directory.mix = function (subject, mixin) {
        var _this = this;
        mixin.children.forEach(function (mixChild) {
            var subjChild = subject.children.find(function (_a) {
                var name = _a.name;
                return name === mixChild.name;
            });
            if (isFile(mixChild)) {
                if (mixChild.content === undefined) {
                    throw new Error('file not loaded : ' + mixChild.fullPath);
                }
                if (subjChild) {
                    if (isFile(subjChild)) {
                        subjChild.content = mixChild.content;
                    }
                    else if (isDir(subjChild)) {
                        throw new Error("can't override directory with file : " + mixChild.fullPath + " , " + subjChild.fullPath);
                    }
                }
                else {
                    subject.children.push(new File(mixChild.name, utils_1.normalizePathNodes([subject.fullPath, mixChild.name]), mixChild.content));
                }
            }
            else if (isDir(mixChild)) {
                if (subjChild) {
                    if (isFile(subjChild)) {
                        throw new Error("can't file directory with directory : " + mixChild.fullPath + " , " + subjChild.fullPath);
                    }
                    else if (isDir(subjChild)) {
                        _this.mix(subjChild, mixChild);
                    }
                }
                else {
                    subject.children.push(Directory.clone(mixChild, [subject.fullPath, mixChild.name]));
                }
            }
        });
        return subject;
    };
    return Directory;
}());
exports.Directory = Directory;
var File = /** @class */ (function () {
    function File(name, fullPath, content) {
        this.name = name;
        this.fullPath = fullPath;
        this.type = 'file';
        if (content)
            this.content = content;
    }
    return File;
}());
exports.File = File;
function isFile(node) {
    if (!node)
        return false;
    return node.type === 'file';
}
exports.isFile = isFile;
function isDir(node) {
    if (!node)
        return false;
    return node.type === 'dir';
}
exports.isDir = isDir;
//# sourceMappingURL=model.js.map