import { Directory, DirectoryContent, File, ShallowDirectory, SimpleStats } from "./model";
import { Correlation, EventEmitter, FileSystem, FileSystemReadSync } from "./api";
export declare namespace NoFeedbackEventsFileSystem {
    type Options = {
        delayEvents: number;
        correlationWindow: number;
    };
}
export declare class NoFeedbackEventsFileSystem implements FileSystem {
    private fs;
    private options;
    baseUrl: string;
    private readonly eventsManager;
    readonly events: EventEmitter;
    private readonly correlateOnce;
    private readonly correlateByWindow;
    constructor(fs: FileSystem, options?: NoFeedbackEventsFileSystem.Options);
    saveFile(fullPath: string, newContent: string, correlation?: Correlation): Promise<Correlation>;
    deleteFile(fullPath: string, correlation?: Correlation): Promise<Correlation>;
    deleteDirectory(fullPath: string, recursive?: boolean, correlation?: Correlation): Promise<Correlation>;
    ensureDirectory(fullPath: string, correlation?: Correlation): Promise<Correlation>;
    loadTextFile(fullPath: string): Promise<string>;
    loadDirectoryTree(fullPath?: string): Promise<Directory>;
    loadDirectoryChildren(fullPath: string): Promise<(File | ShallowDirectory)[]>;
    stat(fullPath: string): Promise<SimpleStats>;
    protected registerCorrelation(correlation: Correlation, once: boolean): string;
}
export declare class NoFeedbackEventsFileSystemSync extends NoFeedbackEventsFileSystem implements FileSystemReadSync {
    private syncFs;
    constructor(syncFs: FileSystemReadSync);
    loadTextFileSync(fullPath: string): string;
    loadDirectoryTreeSync(fullPath?: string | undefined): Directory;
    loadDirectoryChildrenSync(fullPath: string): (File | ShallowDirectory)[];
    loadDirectoryContentSync(fullPath?: string): DirectoryContent;
    statSync(fullPath: string): SimpleStats;
}
