"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var api_1 = require("./api");
var events_manager_1 = require("./events-manager");
var utils_1 = require("./utils");
var NoFeedbackEventsFileSystem = /** @class */ (function () {
    function NoFeedbackEventsFileSystem(fs, options) {
        if (options === void 0) { options = {
            delayEvents: 1,
            correlationWindow: 10000
        }; }
        var _this = this;
        this.fs = fs;
        this.options = options;
        this.eventsManager = new events_manager_1.EventsManager({ delay: this.options.delayEvents });
        this.events = this.eventsManager.events;
        this.correlateOnce = new Set();
        this.correlateByWindow = new Set();
        this.baseUrl = fs.baseUrl;
        this.eventsManager.addEventHandler({
            types: api_1.fileSystemEventNames,
            filter: function (e) {
                return !!e.correlation && (_this.correlateByWindow.has(e.correlation) || _this.correlateOnce.delete(e.correlation));
            },
            apply: function () {
            },
        });
        var emit = this.eventsManager.emit.bind(this.eventsManager);
        api_1.fileSystemEventNames.forEach(function (type) { return _this.fs.events.on(type, emit); });
    }
    NoFeedbackEventsFileSystem.prototype.saveFile = function (fullPath, newContent, correlation) {
        if (correlation === void 0) { correlation = utils_1.makeCorrelationId(); }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.registerCorrelation(correlation, false);
                        return [4 /*yield*/, this.fs.saveFile(fullPath, newContent, correlation)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    NoFeedbackEventsFileSystem.prototype.deleteFile = function (fullPath, correlation) {
        if (correlation === void 0) { correlation = utils_1.makeCorrelationId(); }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.registerCorrelation(correlation, true);
                        return [4 /*yield*/, this.fs.deleteFile(fullPath, correlation)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    NoFeedbackEventsFileSystem.prototype.deleteDirectory = function (fullPath, recursive, correlation) {
        if (correlation === void 0) { correlation = utils_1.makeCorrelationId(); }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.registerCorrelation(correlation, !recursive);
                        return [4 /*yield*/, this.fs.deleteDirectory(fullPath, recursive, correlation)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    NoFeedbackEventsFileSystem.prototype.ensureDirectory = function (fullPath, correlation) {
        if (correlation === void 0) { correlation = utils_1.makeCorrelationId(); }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.registerCorrelation(correlation, false);
                        return [4 /*yield*/, this.fs.ensureDirectory(fullPath, correlation)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    NoFeedbackEventsFileSystem.prototype.loadTextFile = function (fullPath) {
        return this.fs.loadTextFile(fullPath);
    };
    NoFeedbackEventsFileSystem.prototype.loadDirectoryTree = function (fullPath) {
        return this.fs.loadDirectoryTree(fullPath);
    };
    NoFeedbackEventsFileSystem.prototype.loadDirectoryChildren = function (fullPath) {
        return this.fs.loadDirectoryChildren(fullPath);
    };
    NoFeedbackEventsFileSystem.prototype.stat = function (fullPath) {
        return this.fs.stat(fullPath);
    };
    NoFeedbackEventsFileSystem.prototype.registerCorrelation = function (correlation, once) {
        var targetSet = once ? this.correlateOnce : this.correlateByWindow;
        targetSet.add(correlation);
        setTimeout(function () { return targetSet.delete(correlation); }, this.options.correlationWindow);
        return correlation;
    };
    return NoFeedbackEventsFileSystem;
}());
exports.NoFeedbackEventsFileSystem = NoFeedbackEventsFileSystem;
var NoFeedbackEventsFileSystemSync = /** @class */ (function (_super) {
    tslib_1.__extends(NoFeedbackEventsFileSystemSync, _super);
    function NoFeedbackEventsFileSystemSync(syncFs) {
        var _this = _super.call(this, syncFs) || this;
        _this.syncFs = syncFs;
        return _this;
    }
    NoFeedbackEventsFileSystemSync.prototype.loadTextFileSync = function (fullPath) {
        return this.syncFs.loadTextFileSync(fullPath);
    };
    NoFeedbackEventsFileSystemSync.prototype.loadDirectoryTreeSync = function (fullPath) {
        return this.syncFs.loadDirectoryTreeSync(fullPath);
    };
    NoFeedbackEventsFileSystemSync.prototype.loadDirectoryChildrenSync = function (fullPath) {
        return this.syncFs.loadDirectoryChildrenSync(fullPath);
    };
    NoFeedbackEventsFileSystemSync.prototype.loadDirectoryContentSync = function (fullPath) {
        if (fullPath === void 0) { fullPath = ''; }
        return this.syncFs.loadDirectoryContentSync(fullPath);
    };
    NoFeedbackEventsFileSystemSync.prototype.statSync = function (fullPath) {
        return this.syncFs.statSync(fullPath);
    };
    return NoFeedbackEventsFileSystemSync;
}(NoFeedbackEventsFileSystem));
exports.NoFeedbackEventsFileSystemSync = NoFeedbackEventsFileSystemSync;
//# sourceMappingURL=no-feedback-events-fs.js.map