"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
function delayedPromise(delay) {
    return new Promise(function (resolve) { return setTimeout(resolve, delay); });
}
exports.delayedPromise = delayedPromise;
function timeoutPromise(promise, ms, message) {
    if (message === void 0) { message = "timed out after " + ms + "ms"; }
    return new Promise(function (resolve, reject) {
        setTimeout(function () { return reject(new Error(message)); }, ms);
        promise.then(resolve, reject);
    });
}
exports.timeoutPromise = timeoutPromise;
var never = new Promise(function () { return void 0; });
function retryPromise(promiseProvider, _a) {
    var interval = _a.interval, retries = _a.retries, timeout = _a.timeout, _b = _a.timeoutMessage, timeoutMessage = _b === void 0 ? "timed out after " + timeout + "ms" : _b;
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var raceWithTimeout, iterations, lastError, e_1;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    if (!(!timeout && !retries)) return [3 /*break*/, 2];
                    return [4 /*yield*/, promiseProvider()];
                case 1: return [2 /*return*/, _c.sent()];
                case 2:
                    if (timeout && timeout <= retries * interval) {
                        return [2 /*return*/, Promise.reject(new Error("timeout (" + timeout + "ms) must be greater than retries (" + retries + ") times interval (" + interval + "ms)"))];
                    }
                    raceWithTimeout = [never, never];
                    if (timeout) {
                        raceWithTimeout[1] = delayedPromise(timeout).then(function () { return Promise.reject(new Error(timeoutMessage)); });
                    }
                    iterations = retries ? retries + 1 : Number.MAX_SAFE_INTEGER;
                    lastError = null;
                    _c.label = 3;
                case 3:
                    if (!(iterations-- > 0)) return [3 /*break*/, 9];
                    _c.label = 4;
                case 4:
                    _c.trys.push([4, 6, , 7]);
                    raceWithTimeout[0] = promiseProvider();
                    return [4 /*yield*/, Promise.race(raceWithTimeout)];
                case 5: return [2 /*return*/, _c.sent()];
                case 6:
                    e_1 = _c.sent();
                    if (e_1.message === timeoutMessage) {
                        throw lastError || e_1;
                    }
                    lastError = e_1;
                    return [3 /*break*/, 7];
                case 7: return [4 /*yield*/, new Promise(function (resolve) { return setTimeout(resolve, interval); })];
                case 8:
                    _c.sent();
                    return [3 /*break*/, 3];
                case 9: throw lastError || new Error(timeoutMessage);
            }
        });
    });
}
exports.retryPromise = retryPromise;
//# sourceMappingURL=promise-utils.js.map