"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var chai_1 = require("chai");
var events_matcher_1 = require("./events-matcher");
var slow_fs_1 = require("./slow-fs");
var universal_1 = require("../src/universal");
var implementation_suite_1 = require("./implementation-suite");
var sinon_1 = require("sinon");
var existingFileName = 'existing_' + implementation_suite_1.fileName;
var existingDirName = 'existing_' + implementation_suite_1.dirName;
function testCacheReadMethod(options, methodName, args) {
    var _this = this;
    it(methodName + "(" + args.map(function (a) { return JSON.stringify(a); }).join(', ') + ") reflects underlying fs and uses cache on second call", function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var original, fs, originalMethod, cacheMethod, spyMethod, _a, _b, _c, _d, _e, _f, _g;
        return tslib_1.__generator(this, function (_h) {
            switch (_h.label) {
                case 0:
                    original = new universal_1.MemoryFileSystem('', {
                        content: (_g = {},
                            _g[existingDirName] = {},
                            _g[existingFileName] = 'foo',
                            _g)
                    });
                    fs = new universal_1.CacheFileSystem(original, options);
                    originalMethod = original[methodName].bind(original);
                    cacheMethod = fs[methodName].bind(fs);
                    spyMethod = sinon_1.spy(originalMethod);
                    original[methodName] = spyMethod;
                    _b = chai_1.expect;
                    return [4 /*yield*/, cacheMethod.apply(void 0, args)];
                case 1:
                    _c = (_a = _b.apply(void 0, [_h.sent()]).to).eql;
                    return [4 /*yield*/, originalMethod.apply(void 0, args)];
                case 2:
                    _c.apply(_a, [_h.sent()]);
                    chai_1.expect(spyMethod).to.have.callCount(1);
                    spyMethod.resetHistory();
                    _e = chai_1.expect;
                    return [4 /*yield*/, cacheMethod.apply(void 0, args)];
                case 3:
                    _f = (_d = _e.apply(void 0, [_h.sent()]).to).eql;
                    return [4 /*yield*/, originalMethod.apply(void 0, args)];
                case 4:
                    _f.apply(_d, [_h.sent()]);
                    chai_1.expect(spyMethod).to.have.callCount(0);
                    return [2 /*return*/];
            }
        });
    }); });
}
describe("the cache file system proxy", function () {
    var eventMatcherOptions = { retries: 15, interval: 2, timeout: 40, noExtraEventsGrace: 10 };
    implementation_suite_1.assertFileSystemContract(function () { return tslib_1.__awaiter(_this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
        return [2 /*return*/, new universal_1.CacheFileSystem(new universal_1.MemoryFileSystem())];
    }); }); }, eventMatcherOptions);
    implementation_suite_1.assertFileSystemSyncContract(function () { return tslib_1.__awaiter(_this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
        return [2 /*return*/, new universal_1.CacheFileSystem(new universal_1.MemoryFileSystem())];
    }); }); }, eventMatcherOptions);
    describe("using slow FileSystem (performance acceptance test)", function () {
        var timeout = 200;
        var fs;
        var slow;
        var startTimestamp;
        var matcher;
        beforeEach(function () {
            startTimestamp = Date.now();
            slow = new slow_fs_1.SlowFs(timeout);
            fs = new universal_1.CacheFileSystem(slow);
            matcher = new events_matcher_1.EventsMatcher(eventMatcherOptions);
            matcher.track.apply(matcher, [fs.events].concat(universal_1.fileSystemEventNames));
        });
        it('loads file faster after it has been saved', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, fs.saveFile(implementation_suite_1.fileName, implementation_suite_1.content)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, fs.loadTextFile(implementation_suite_1.fileName)];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, chai_1.expect(Date.now() - startTimestamp).to.be.lessThan(timeout * 2)];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
        it('loads file faster after it has been saved from outside', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _this = this;
            var onFileCreated, _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        onFileCreated = new Promise(function (resolve) {
                            fs.events.once('fileCreated', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                return tslib_1.__generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0: return [4 /*yield*/, fs.loadTextFile(implementation_suite_1.fileName)];
                                        case 1:
                                            _a.sent();
                                            resolve(Date.now() - startTimestamp);
                                            return [2 /*return*/];
                                    }
                                });
                            }); });
                        });
                        slow.saveFile(implementation_suite_1.fileName, implementation_suite_1.content);
                        _a = chai_1.expect;
                        return [4 /*yield*/, onFileCreated];
                    case 1: return [2 /*return*/, _a.apply(void 0, [_b.sent()]).to.be.lessThan(timeout * 2)];
                }
            });
        }); });
        it('loads tree faster after it has been loaded before', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, fs.loadDirectoryTree()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, fs.loadDirectoryTree()];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, chai_1.expect(Date.now() - startTimestamp).to.be.lessThan(timeout * 2)];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
        it('stat directory faster after it has been cached before', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, fs.ensureDirectory(implementation_suite_1.dirName)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, fs.stat(implementation_suite_1.dirName)];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, chai_1.expect(Date.now() - startTimestamp).to.be.lessThan(timeout * 2)];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
        it('stat file faster after stat has run on it before', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, fs.saveFile(implementation_suite_1.fileName, implementation_suite_1.content)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, fs.stat(implementation_suite_1.fileName)];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, chai_1.expect(Date.now() - startTimestamp).to.be.lessThan(timeout * 2)];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
    });
    testCacheReadMethod({}, 'loadTextFile', [existingFileName]);
    testCacheReadMethod({}, 'loadDirectoryChildren', ['']);
    testCacheReadMethod({}, 'loadDirectoryChildren', [existingDirName]);
    testCacheReadMethod({}, 'loadDirectoryTree', []);
    testCacheReadMethod({}, 'loadDirectoryTree', [existingDirName]);
    testCacheReadMethod({}, 'stat', [existingFileName]);
    testCacheReadMethod({}, 'stat', [existingDirName]);
    describe("with propagateSyncRead = true", function () {
        var options = { propagateSyncRead: true };
        it('breaks if not provided with sync file system', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var nonSyncFs;
            return tslib_1.__generator(this, function (_a) {
                nonSyncFs = new slow_fs_1.SlowFs(1);
                chai_1.expect(function () { return new universal_1.CacheFileSystem(nonSyncFs, options); }).to.throw(Error);
                return [2 /*return*/];
            });
        }); });
        testCacheReadMethod(options, 'loadTextFileSync', [existingFileName]);
        testCacheReadMethod(options, 'loadDirectoryChildrenSync', ['']);
        testCacheReadMethod(options, 'loadDirectoryChildrenSync', [existingDirName]);
        testCacheReadMethod(options, 'loadDirectoryTreeSync', []);
        testCacheReadMethod(options, 'loadDirectoryTreeSync', [existingDirName]);
        testCacheReadMethod(options, 'statSync', [existingFileName]);
        testCacheReadMethod(options, 'statSync', [existingDirName]);
        testCacheReadMethod(options, 'loadDirectoryContentSync', []);
        testCacheReadMethod(options, 'loadDirectoryContentSync', [existingDirName]);
    });
    describe("unexpected error behavior", function () {
        var fs;
        var original;
        var matcher;
        beforeEach(function () {
            original = new universal_1.MemoryFileSystem();
            fs = new universal_1.CacheFileSystem(original);
            matcher = new events_matcher_1.EventsMatcher({
                retries: 30,
                interval: 5,
                noExtraEventsGrace: 150,
                timeout: 300
            });
            matcher.track.apply(matcher, [fs.events].concat(universal_1.fileSystemEventNames));
        });
        it('emits `unexpectedError` when reSyncOnError option is false', function () {
            var fs = new universal_1.CacheFileSystem(original, { reSyncOnError: false });
            var matcher = new events_matcher_1.EventsMatcher(eventMatcherOptions);
            matcher.track.apply(matcher, [fs.events].concat(universal_1.fileSystemEventNames));
            original.events.emit('unexpectedError', { type: 'unexpectedError' });
            return matcher.expect([{ type: 'unexpectedError' }]);
        });
        describe('recovering from missed changes in underlying filesystem', function () {
            beforeEach('turn off notifications from underlying fs and set up cache', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            original.emit = function () { return false; };
                            return [4 /*yield*/, fs.ensureDirectory(existingDirName)];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, fs.loadDirectoryChildren(existingDirName)];
                        case 2:
                            _a.sent();
                            return [4 /*yield*/, fs.saveFile(existingFileName, 'foo')];
                        case 3:
                            _a.sent();
                            return [4 /*yield*/, matcher.expect([
                                    { type: 'directoryCreated', fullPath: existingDirName },
                                    { type: 'fileCreated', fullPath: existingFileName, newContent: 'foo' }
                                ])];
                        case 4:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            function emitErrorFromUnderlyingFs() {
                original.events.emit('unexpectedError', { type: 'unexpectedError' });
            }
            it('does not emit `fileCreated` if the file was never cached', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, original.saveFile(implementation_suite_1.fileName, implementation_suite_1.content)];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, matcher.expect([])];
                        case 2:
                            _a.sent();
                            emitErrorFromUnderlyingFs();
                            return [4 /*yield*/, matcher.expect([])];
                        case 3:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            it('emits `fileChanged` if the file was previously cached', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, original.saveFile(existingFileName, implementation_suite_1.content)];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, matcher.expect([])];
                        case 2:
                            _a.sent();
                            emitErrorFromUnderlyingFs();
                            return [4 /*yield*/, matcher.expect([{ type: 'fileChanged', fullPath: existingFileName, newContent: implementation_suite_1.content }])];
                        case 3:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            it('emits `fileDeleted` if there is cached file and no real file after error', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, original.deleteFile(existingFileName)];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, matcher.expect([])];
                        case 2:
                            _a.sent();
                            emitErrorFromUnderlyingFs();
                            return [4 /*yield*/, matcher.expect([{ type: 'fileDeleted', fullPath: existingFileName }])];
                        case 3:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            it('emits `directoryCreated` if there is not cached dir after error', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, original.ensureDirectory(existingDirName + '/' + implementation_suite_1.dirName)];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, matcher.expect([])];
                        case 2:
                            _a.sent();
                            emitErrorFromUnderlyingFs();
                            return [4 /*yield*/, matcher.expect([{ type: 'directoryCreated', fullPath: existingDirName + '/' + implementation_suite_1.dirName }])];
                        case 3:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            it('emits `directoryDeleted` if there is cached dir and no real dir after error', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, original.deleteDirectory(existingDirName)];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, matcher.expect([])];
                        case 2:
                            _a.sent();
                            emitErrorFromUnderlyingFs();
                            return [4 /*yield*/, matcher.expect([{ type: 'directoryDeleted', fullPath: existingDirName }])];
                        case 3:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
        });
    });
    describe("lazyness", function () {
        var fs;
        var original;
        var matcher;
        beforeEach(function () {
            original = new universal_1.MemoryFileSystem('', {
                content: {
                    foo: {}
                }
            });
            fs = new universal_1.CacheFileSystem(original);
            matcher = new events_matcher_1.EventsMatcher({
                retries: 30,
                interval: 5,
                noExtraEventsGrace: 150,
                timeout: 300
            });
            matcher.track.apply(matcher, [fs.events].concat(universal_1.fileSystemEventNames));
        });
        it('does not load underlying fs tree more than needs to', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                sinon_1.spy(original, 'loadDirectoryTree');
                fs.loadDirectoryTree('foo');
                chai_1.expect(original.loadDirectoryTree).to.have.callCount(1);
                chai_1.expect(original.loadDirectoryTree).to.have.been.calledWithExactly('foo');
                return [2 /*return*/];
            });
        }); });
        it('does not load underlying fs tree more than needs to', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                sinon_1.spy(original, 'loadDirectoryChildren');
                fs.loadDirectoryChildren('foo');
                chai_1.expect(original.loadDirectoryChildren).to.have.callCount(1);
                chai_1.expect(original.loadDirectoryChildren).to.have.been.calledWithExactly('foo');
                return [2 /*return*/];
            });
        }); });
    });
});
//# sourceMappingURL=cache-fs.spec.js.map