"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var events_manager_1 = require("../src/events-manager");
var events_matcher_1 = require("./events-matcher");
var universal_1 = require("../src/universal");
var promise_utils_1 = require("../src/promise-utils");
var sinon = require("sinon");
var chai_1 = require("chai");
describe('EventsManager', function () {
    var matcher;
    var em;
    var event1 = { type: 'fileDeleted', fullPath: 'foo' };
    var event2 = { type: 'fileDeleted', fullPath: 'bar' };
    var event3 = { type: 'directoryCreated', fullPath: 'baz' };
    var handler = {
        types: ['fileDeleted'],
        filter: function (e) { return e === event1; },
        apply: function () { return event3; },
    };
    beforeEach('setup', function () {
        em = new events_manager_1.EventsManager();
        matcher = new events_matcher_1.EventsMatcher({ retries: 15, interval: 2, timeout: 40, noExtraEventsGrace: 10 });
        matcher.track.apply(matcher, [em.events].concat(universal_1.fileSystemEventNames));
    });
    describe('delays', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var _this = this;
        var delay, delayedManager, delayedManagerMatcher;
        return tslib_1.__generator(this, function (_a) {
            delay = 50;
            beforeEach('setup', function () {
                em = new events_manager_1.EventsManager();
                delayedManager = new events_manager_1.EventsManager({ delay: delay });
                delayedManagerMatcher = new events_matcher_1.EventsMatcher({
                    retries: 15,
                    interval: 2,
                    timeout: 40,
                    noExtraEventsGrace: 10
                });
                delayedManagerMatcher.track.apply(delayedManagerMatcher, [delayedManager.events].concat(universal_1.fileSystemEventNames));
            });
            it('events', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            delayedManager.emit(event1); // this will change
                            return [4 /*yield*/, delayedManagerMatcher.expect([])];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, promise_utils_1.delayedPromise(delay * 2)];
                        case 2:
                            _a.sent();
                            return [4 /*yield*/, delayedManagerMatcher.expect([event1])];
                        case 3:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            it('handlers', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                var spy;
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            spy = sinon.spy();
                            delayedManager.addEventHandler({
                                types: ['fileDeleted'],
                                filter: spy,
                                apply: function () {
                                },
                            });
                            delayedManager.emit(event1); // this will change
                            return [4 /*yield*/, chai_1.expect(spy).to.have.callCount(0)];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, promise_utils_1.delayedPromise(delay * 2)];
                        case 2:
                            _a.sent();
                            return [4 /*yield*/, chai_1.expect(spy).to.have.callCount(1)];
                        case 3:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            return [2 /*return*/];
        });
    }); });
    it('emits events', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    em.emit(event1);
                    return [4 /*yield*/, matcher.expect([event1])];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it('modifies matching events', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    em.addEventHandler(handler);
                    em.emit(event1); // this will change
                    em.emit(event2); // this does not match handler
                    return [4 /*yield*/, matcher.expect([event3, event2])];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it('filters matching events', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    em.addEventHandler({
                        types: ['fileDeleted'],
                        filter: function (e) { return e === event1; },
                        apply: function () {
                        },
                    });
                    em.emit(event1); // this will be filtered
                    em.emit(event2); // this does not match handler
                    return [4 /*yield*/, matcher.expect([event2])];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it('respects time out (assume clean-up)', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    em.addEventHandler(handler, 1);
                    return [4 /*yield*/, promise_utils_1.delayedPromise(10)];
                case 1:
                    _a.sent();
                    em.emit(event1);
                    return [4 /*yield*/, matcher.expect([event1])];
                case 2:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it('removes handlers', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    em.addEventHandler(handler);
                    em.removeEventHandler(handler);
                    em.emit(event1);
                    return [4 /*yield*/, matcher.expect([event1])];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
});
//# sourceMappingURL=events-manager.spec.js.map