"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var tmp_1 = require("tmp");
var fs_1 = require("fs");
var path_1 = require("path");
var implementation_suite_1 = require("./implementation-suite");
var events_matcher_1 = require("./events-matcher");
var nodejs_1 = require("../src/nodejs");
var no_feedback_events_fs_1 = require("../src/no-feedback-events-fs");
var promise_utils_1 = require("../src/promise-utils");
var eventMatcherOptions = {
    retries: 20,
    interval: 25,
    timeout: 1000,
    noExtraEventsGrace: 150
};
describe("integration of the local filesystem with nofeedbackFS", function () {
    var dirCleanup;
    var rootPath;
    var testPath;
    var counter = 0;
    var disposableFileSystem;
    before(function (done) {
        tmp_1.dir({ unsafeCleanup: true }, function (_err, path, cleanupCallback) {
            dirCleanup = cleanupCallback;
            rootPath = path;
            done();
        });
    });
    after(function () {
        try {
            dirCleanup();
        }
        catch (e) {
            console.log('cleanup error', e);
        }
    });
    var fs;
    var matcher;
    beforeEach(function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    matcher = new events_matcher_1.EventsMatcher(eventMatcherOptions);
                    return [4 /*yield*/, getFS()];
                case 1:
                    fs = _a.sent();
                    matcher.track.apply(matcher, [fs.events].concat(nodejs_1.fileSystemEventNames));
                    return [2 /*return*/];
            }
        });
    }); });
    afterEach(function () {
        // if beforeEach fails, disposableFileSystem can stay undefined
        disposableFileSystem && disposableFileSystem.dispose();
    });
    function getFS() {
        testPath = path_1.join(rootPath, 'fs_' + (counter++));
        fs_1.mkdirSync(testPath);
        disposableFileSystem = new nodejs_1.LocalFileSystem(testPath);
        return disposableFileSystem.init();
    }
    it('should not provide feedback when bombarding changes (stress)', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var path, expectedChangeEvents, noFeed, nofeedMatcher, i, e_1, e2_1;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    path = path_1.join(testPath, implementation_suite_1.fileName);
                    expectedChangeEvents = [];
                    fs_1.writeFileSync(path, implementation_suite_1.content);
                    return [4 /*yield*/, matcher.expect([{ type: 'fileCreated', fullPath: implementation_suite_1.fileName, newContent: implementation_suite_1.content }])];
                case 1:
                    _a.sent();
                    // this is a magical fix for test flakyness. let the underlying FS calm before bombarding with changes.
                    return [4 /*yield*/, promise_utils_1.delayedPromise(100)];
                case 2:
                    // this is a magical fix for test flakyness. let the underlying FS calm before bombarding with changes.
                    _a.sent();
                    noFeed = new no_feedback_events_fs_1.NoFeedbackEventsFileSystem(fs, { delayEvents: 1, correlationWindow: 10000 });
                    nofeedMatcher = new events_matcher_1.EventsMatcher({
                        alwaysExpectEmpty: true,
                        noExtraEventsGrace: 1000,
                        interval: 100,
                        retries: 40,
                        timeout: 1000
                    });
                    nofeedMatcher.track.apply(nofeedMatcher, [noFeed.events].concat(nodejs_1.fileSystemEventNames));
                    i = 1;
                    _a.label = 3;
                case 3:
                    if (!(i < 200)) return [3 /*break*/, 6];
                    return [4 /*yield*/, promise_utils_1.delayedPromise(1)];
                case 4:
                    _a.sent();
                    noFeed.saveFile(implementation_suite_1.fileName, 'content:' + i, '' + i);
                    expectedChangeEvents.push({
                        type: 'fileChanged',
                        fullPath: implementation_suite_1.fileName,
                        newContent: 'content:' + i,
                        correlation: '' + i
                    });
                    _a.label = 5;
                case 5:
                    i++;
                    return [3 /*break*/, 3];
                case 6:
                    _a.trys.push([6, 8, , 13]);
                    return [4 /*yield*/, nofeedMatcher.expect([])];
                case 7:
                    _a.sent();
                    return [3 /*break*/, 13];
                case 8:
                    e_1 = _a.sent();
                    console.error('nofeedMatcher failed. printing underlying events');
                    _a.label = 9;
                case 9:
                    _a.trys.push([9, 11, , 12]);
                    return [4 /*yield*/, matcher.expect(expectedChangeEvents)];
                case 10:
                    _a.sent();
                    return [3 /*break*/, 12];
                case 11:
                    e2_1 = _a.sent();
                    console.error(e2_1);
                    return [3 /*break*/, 12];
                case 12: throw e_1;
                case 13: return [2 /*return*/];
            }
        });
    }); });
});
//# sourceMappingURL=local-fs-no-feedback.spec.node.js.map