"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var implementation_suite_1 = require("./implementation-suite");
var universal_1 = require("../src/universal");
var chai_1 = require("chai");
var content = {
    "a.file": "hello",
    "src": {
        "a.ts": "a",
        "b.js": "b",
        "nested": {
            "file.zag": "nested-file"
        }
    }
};
function assertContent(fs) {
    chai_1.expect(fs.loadTextFileSync('a.file')).to.be.equal('hello');
    chai_1.expect(fs.loadTextFileSync('src/a.ts')).to.be.equal('a');
    chai_1.expect(fs.loadTextFileSync('src/b.js')).to.be.equal('b');
    chai_1.expect(fs.loadTextFileSync('src/nested/file.zag')).to.be.equal('nested-file');
}
describe("the in-memory implementation", function () {
    var _this = this;
    implementation_suite_1.assertFileSystemContract(function () { return Promise.resolve(new universal_1.MemoryFileSystem()); }, { retries: 15, interval: 2, timeout: 40, noExtraEventsGrace: 10 });
    implementation_suite_1.assertFileSystemSyncContract(function () { return Promise.resolve(new universal_1.MemoryFileSystem()); }, { retries: 15, interval: 2, timeout: 40, noExtraEventsGrace: 10 });
    describe("constructor options", function () {
        it("content sets initial content", function () {
            var fs = new universal_1.MemoryFileSystem(undefined, { content: content });
            assertContent(fs);
        });
        it("model sets initial content", function () {
            var model = universal_1.Directory.fromContent(content);
            var fs = new universal_1.MemoryFileSystem(undefined, { model: model });
            assertContent(fs);
        });
        it("model sets internal live model", function () {
            var newFilePath = 'foo/bar/file';
            var newContent = 'new file';
            var model = universal_1.Directory.fromContent(content);
            new universal_1.MemoryFileSystem(undefined, { model: model }).saveFile(newFilePath, newContent);
            chai_1.expect(model).to.not.eql(universal_1.Directory.fromContent(content));
            chai_1.expect(new universal_1.MemoryFileSystem(undefined, { model: model }).loadTextFileSync(newFilePath)).to.be.equal(newContent);
        });
    });
    describe('static addContent()', function () {
        it('adds content to an existing memory files system', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var fs, newContent;
            return tslib_1.__generator(this, function (_a) {
                fs = new universal_1.MemoryFileSystem('', { content: { "a.file": 'hello' } });
                universal_1.MemoryFileSystem.addContent(fs, content.src, 'src');
                newContent = fs.loadDirectoryContentSync();
                // a trick to structural equality with no regards to ordering in arrays
                // (a contains b && b contains a) === a equals b
                chai_1.expect(newContent).to.containSubset(content);
                chai_1.expect(content).to.containSubset(newContent);
                return [2 /*return*/];
            });
        }); });
    });
});
//# sourceMappingURL=memory-fs.spec.js.map