"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var universal_1 = require("../src/universal");
var chai_1 = require("chai");
var emptyContent = {};
var content = {
    "a.file": "hello",
    "src": {
        "a.ts": "a",
        "b.js": "b",
        "nested": {
            "file.zag": "nested-file"
        }
    }
};
var mixinContent = {
    "b.file": "hello",
    "src": {
        "nested": {
            "file2.zag": "new file"
        }
    }
};
var mixedContent = {
    "a.file": "hello",
    "b.file": "hello",
    "src": {
        "a.ts": "a",
        "b.js": "b",
        "nested": {
            "file.zag": "nested-file",
            "file2.zag": "new file"
        }
    }
};
describe('model', function () {
    describe('Directory.fromContent', function () {
        it('works on empty input', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var emptyDirectory;
            return tslib_1.__generator(this, function (_a) {
                emptyDirectory = {
                    "fullPath": "",
                    "name": "",
                    "type": "dir",
                    "children": []
                };
                chai_1.expect(universal_1.Directory.fromContent(emptyContent)).to.eql(emptyDirectory);
                return [2 /*return*/];
            });
        }); });
        it('works on input with data', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var contentAsDirectory;
            return tslib_1.__generator(this, function (_a) {
                contentAsDirectory = {
                    "fullPath": "",
                    "name": "",
                    "type": "dir",
                    "children": [{
                            "content": "hello",
                            "fullPath": "a.file",
                            "name": "a.file",
                            "type": "file"
                        }, {
                            "children": [{
                                    "content": "a",
                                    "fullPath": "src/a.ts",
                                    "name": "a.ts",
                                    "type": "file",
                                }, {
                                    "content": "b",
                                    "fullPath": "src/b.js",
                                    "name": "b.js",
                                    "type": "file",
                                }, {
                                    "children": [{
                                            "content": "nested-file",
                                            "fullPath": "src/nested/file.zag",
                                            "name": "file.zag",
                                            "type": "file",
                                        }],
                                    "fullPath": "src/nested",
                                    "name": "nested",
                                    "type": "dir",
                                }],
                            "fullPath": "src",
                            "name": "src",
                            "type": "dir"
                        }]
                };
                chai_1.expect(universal_1.Directory.fromContent(content)).to.eql(contentAsDirectory);
                return [2 /*return*/];
            });
        }); });
        it('can override name', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.fromContent(content, 'foo')).to.eql(universal_1.Directory.fromContent({ foo: content }).children.find(function (c) { return c.name === 'foo'; }));
                return [2 /*return*/];
            });
        }); });
        it('can override name and location', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var contentAsDirectory;
            return tslib_1.__generator(this, function (_a) {
                contentAsDirectory = {
                    "children": [{
                            "content": "hello",
                            "fullPath": "bar/baz/foo/a.file",
                            "name": "a.file",
                            "type": "file"
                        }, {
                            "children": [{
                                    "content": "a",
                                    "fullPath": "bar/baz/foo/src/a.ts",
                                    "name": "a.ts",
                                    "type": "file"
                                }, {
                                    "content": "b",
                                    "fullPath": "bar/baz/foo/src/b.js",
                                    "name": "b.js",
                                    "type": "file"
                                }, {
                                    "children": [{
                                            "content": "nested-file",
                                            "fullPath": "bar/baz/foo/src/nested/file.zag",
                                            "name": "file.zag",
                                            "type": "file"
                                        }],
                                    "fullPath": "bar/baz/foo/src/nested",
                                    "name": "nested",
                                    "type": "dir",
                                }
                            ],
                            "fullPath": "bar/baz/foo/src",
                            "name": "src",
                            "type": "dir"
                        }
                    ],
                    "fullPath": "bar/baz/foo",
                    "name": "foo",
                    "type": "dir"
                };
                chai_1.expect(universal_1.Directory.fromContent(content, 'foo', 'bar/baz')).to.eql(contentAsDirectory);
                return [2 /*return*/];
            });
        }); });
    });
    describe('Directory.getSubDir', function () {
        it('works on empty input and empty path', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.getSubDir(universal_1.Directory.fromContent({}), '')).to.eql(universal_1.Directory.fromContent({}));
                return [2 /*return*/];
            });
        }); });
        it('works on input with data and empty path', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.getSubDir(universal_1.Directory.fromContent(content), '')).to.eql(universal_1.Directory.fromContent(content));
                return [2 /*return*/];
            });
        }); });
        it('works on input with data and real path', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.getSubDir(universal_1.Directory.fromContent(content), 'src/nested')).to.eql(universal_1.Directory.fromContent(content.src.nested, 'nested', 'src'));
                return [2 /*return*/];
            });
        }); });
    });
    describe('Directory.toContent', function () {
        it('works on empty input', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.toContent(universal_1.Directory.fromContent({}))).to.eql(emptyContent);
                return [2 /*return*/];
            });
        }); });
        it('works on input with data', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.toContent(universal_1.Directory.fromContent(content))).to.eql(content);
                return [2 /*return*/];
            });
        }); });
    });
    describe('Directory.clone', function () {
        it('works on empty input', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.clone(universal_1.Directory.fromContent({}))).to.eql(universal_1.Directory.fromContent({}));
                return [2 /*return*/];
            });
        }); });
        it('works on input with data', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.clone(universal_1.Directory.fromContent(content))).to.eql(universal_1.Directory.fromContent(content));
                return [2 /*return*/];
            });
        }); });
        it('can override path', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.clone(universal_1.Directory.fromContent(content), 'foo')).to.eql(universal_1.Directory.fromContent(content, 'foo'));
                return [2 /*return*/];
            });
        }); });
    });
    describe('Directory.cloneStructure', function () {
        it('works on empty input', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.cloneStructure(universal_1.Directory.fromContent({}))).to.eql(universal_1.Directory.fromContent({}));
                return [2 /*return*/];
            });
        }); });
        it('works on input with data', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var contentAsDirectoryStructure;
            return tslib_1.__generator(this, function (_a) {
                contentAsDirectoryStructure = {
                    "fullPath": "",
                    "name": "",
                    "type": "dir",
                    "children": [{
                            "fullPath": "a.file",
                            "name": "a.file",
                            "type": "file"
                        }, {
                            "children": [{
                                    "fullPath": "src/a.ts",
                                    "name": "a.ts",
                                    "type": "file",
                                }, {
                                    "fullPath": "src/b.js",
                                    "name": "b.js",
                                    "type": "file",
                                }, {
                                    "children": [{
                                            "fullPath": "src/nested/file.zag",
                                            "name": "file.zag",
                                            "type": "file",
                                        }],
                                    "fullPath": "src/nested",
                                    "name": "nested",
                                    "type": "dir",
                                }],
                            "fullPath": "src",
                            "name": "src",
                            "type": "dir"
                        }]
                };
                chai_1.expect(universal_1.Directory.cloneStructure(universal_1.Directory.fromContent(content))).to.eql(contentAsDirectoryStructure);
                return [2 /*return*/];
            });
        }); });
    });
    describe('Directory.mix', function () {
        it('returns subject', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var subject;
            return tslib_1.__generator(this, function (_a) {
                subject = universal_1.Directory.fromContent({});
                chai_1.expect(universal_1.Directory.mix(subject, universal_1.Directory.fromContent({}))).to.equal(subject);
                return [2 /*return*/];
            });
        }); });
        it('works on empty input', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.mix(universal_1.Directory.fromContent({}), universal_1.Directory.fromContent({}))).to.eql(universal_1.Directory.fromContent({}));
                return [2 /*return*/];
            });
        }); });
        it('works on input with data', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.mix(universal_1.Directory.fromContent({}), universal_1.Directory.fromContent(content))).to.eql(universal_1.Directory.fromContent(content));
                return [2 /*return*/];
            });
        }); });
        it('works on subject with same data', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                chai_1.expect(universal_1.Directory.mix(universal_1.Directory.fromContent(content), universal_1.Directory.fromContent(content))).to.eql(universal_1.Directory.fromContent(content));
                return [2 /*return*/];
            });
        }); });
        it('works on mix with additional data', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var mixed, expected;
            return tslib_1.__generator(this, function (_a) {
                mixed = universal_1.Directory.mix(universal_1.Directory.fromContent(content), universal_1.Directory.fromContent(mixinContent));
                expected = universal_1.Directory.fromContent(mixedContent);
                // a trick to structural equality with no regards to ordering in arrays
                // (a contains b && b contains a) === a equals b
                chai_1.expect(mixed).to.containSubset(expected);
                chai_1.expect(expected).to.containSubset(mixed);
                return [2 /*return*/];
            });
        }); });
        it('works on mix with different file content', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var content2, mixed;
            return tslib_1.__generator(this, function (_a) {
                content2 = Object.create(content);
                content2["a.file"] = "new content";
                chai_1.expect(content2, 'false-positive test! content2 should override existing content file').to.not.containSubset(content);
                mixed = universal_1.Directory.mix(universal_1.Directory.fromContent(content), universal_1.Directory.fromContent(content2));
                chai_1.expect(mixed).to.containSubset(universal_1.Directory.fromContent(content2));
                return [2 /*return*/];
            });
        }); });
    });
});
//# sourceMappingURL=model.spec.js.map