"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var universal_1 = require("../src/universal");
var implementation_suite_1 = require("./implementation-suite");
var sinon = require("sinon");
var chai_1 = require("chai");
function proxy(Proxy, externalChanges) {
    var _this = this;
    return function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var innerFs, proxy, hybrid;
        return tslib_1.__generator(this, function (_a) {
            innerFs = new universal_1.MemoryFileSystem();
            proxy = new Proxy(innerFs);
            if (externalChanges) {
                hybrid = Object.create(innerFs);
                hybrid.events = proxy.events;
                return [2 /*return*/, hybrid];
            }
            return [2 /*return*/, proxy];
        });
    }); };
}
describe("the no-feedback-events file system proxy", function () {
    var eventMatcherOptions = {
        retries: 15,
        interval: 2,
        timeout: 40,
        noExtraEventsGrace: 5
    };
    implementation_suite_1.assertFileSystemContract(proxy(universal_1.NoFeedbackEventsFileSystem, false), tslib_1.__assign({}, eventMatcherOptions, { alwaysExpectEmpty: true }));
    describe("external changes", function () {
        implementation_suite_1.assertFileSystemContract(proxy(universal_1.NoFeedbackEventsFileSystem, true), eventMatcherOptions);
    });
    describe("the synchronous proxy", function () {
        implementation_suite_1.assertFileSystemContract(proxy(universal_1.NoFeedbackEventsFileSystemSync, false), tslib_1.__assign({}, eventMatcherOptions, { alwaysExpectEmpty: true }));
        implementation_suite_1.assertFileSystemSyncContract(proxy(universal_1.NoFeedbackEventsFileSystemSync, false), tslib_1.__assign({}, eventMatcherOptions, { alwaysExpectEmpty: true }));
        describe("external changes", function () {
            implementation_suite_1.assertFileSystemContract(proxy(universal_1.NoFeedbackEventsFileSystemSync, true), eventMatcherOptions);
        });
    });
    describe('No feedback simple tests', function () {
        var _this = this;
        it('should not provide feed back', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var memFs, noFeed, spy;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        memFs = new universal_1.MemoryFileSystem();
                        universal_1.MemoryFileSystem.addContent(memFs, {
                            'aFile': 'gaga'
                        });
                        noFeed = new universal_1.NoFeedbackEventsFileSystem(memFs, { delayEvents: 0, correlationWindow: 100 });
                        spy = sinon.spy();
                        noFeed.events.on('fileChanged', spy);
                        return [4 /*yield*/, noFeed.saveFile('aFile', 'baga')];
                    case 1:
                        _a.sent();
                        chai_1.expect(spy.getCalls().length).to.equal(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('should provide feedback for external', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var memFs, noFeed, spy;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        memFs = new universal_1.MemoryFileSystem();
                        universal_1.MemoryFileSystem.addContent(memFs, {
                            'aFile': 'gaga'
                        });
                        noFeed = new universal_1.NoFeedbackEventsFileSystem(memFs, { delayEvents: 0, correlationWindow: 100 });
                        spy = sinon.spy();
                        noFeed.events.on('fileChanged', spy);
                        return [4 /*yield*/, memFs.saveFile('aFile', 'baga')];
                    case 1:
                        _a.sent();
                        chai_1.expect(spy.getCalls().length).to.equal(1);
                        return [2 /*return*/];
                }
            });
        }); });
    });
});
//# sourceMappingURL=no-feedback-events-fs.spec.js.map