"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var api_1 = require("../src/api");
var memory_fs_1 = require("../src/memory-fs");
var promise_utils_1 = require("../src/promise-utils");
var SlowFs = /** @class */ (function () {
    function SlowFs(delay) {
        this.delay = delay;
        this.baseUrl = 'slow';
        this.fs = new memory_fs_1.MemoryFileSystem();
        this.events = this.fs.events;
    }
    SlowFs.prototype.saveFile = function (fullPath, newContent, correlation) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, promise_utils_1.delayedPromise(this.delay)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.fs.saveFile(fullPath, newContent, correlation)];
                }
            });
        });
    };
    SlowFs.prototype.deleteFile = function (fullPath, correlation) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, promise_utils_1.delayedPromise(this.delay)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.fs.deleteFile(fullPath, correlation)];
                }
            });
        });
    };
    SlowFs.prototype.deleteDirectory = function (fullPath, recursive, correlation) {
        if (recursive === void 0) { recursive = false; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, promise_utils_1.delayedPromise(this.delay)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.fs.deleteDirectory(fullPath, recursive, correlation)];
                }
            });
        });
    };
    SlowFs.prototype.ensureDirectory = function (fullPath, correlation) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, promise_utils_1.delayedPromise(this.delay)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.fs.ensureDirectory(fullPath, correlation)];
                }
            });
        });
    };
    SlowFs.prototype.loadTextFile = function (fullPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, promise_utils_1.delayedPromise(this.delay)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.fs.loadTextFile(fullPath)];
                }
            });
        });
    };
    SlowFs.prototype.loadDirectoryTree = function (fullPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, promise_utils_1.delayedPromise(this.delay)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.fs.loadDirectoryTree(fullPath)];
                }
            });
        });
    };
    SlowFs.prototype.loadDirectoryChildren = function (fullPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, promise_utils_1.delayedPromise(this.delay)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.fs.loadDirectoryChildren(fullPath)];
                }
            });
        });
    };
    SlowFs.prototype.stat = function (fullPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, promise_utils_1.delayedPromise(this.delay)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.fs.stat(fullPath)];
                }
            });
        });
    };
    SlowFs.prototype.dispose = function () {
        var _this = this;
        setTimeout(function () { return api_1.isDisposable(_this.fs) && _this.fs.dispose(); }, this.delay);
    };
    return SlowFs;
}());
exports.SlowFs = SlowFs;
//# sourceMappingURL=slow-fs.js.map