"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var universal_1 = require("../src/universal");
var chai_1 = require("chai");
var user_utils_1 = require("../src/user-utils");
var no_feedback_events_fs_1 = require("../src/no-feedback-events-fs");
describe('user-utils', function () {
    describe('checkExistsSync', function () {
        var fs = new universal_1.MemoryFileSystem('', { content: { foo: { bar: { "a.file": 'hello' } } } });
        it('true for existing file', function () {
            chai_1.expect(universal_1.checkExistsSync('file', fs, 'foo/bar/a.file')).to.eql(true);
        });
        it('true for existing dir', function () {
            chai_1.expect(universal_1.checkExistsSync('dir', fs, 'foo/bar')).to.eql(true);
        });
        it('false for non existing file', function () {
            chai_1.expect(universal_1.checkExistsSync('file', fs, 'a/foo/bar/a.file')).to.eql(false);
        });
        it('false for non existing dir', function () {
            chai_1.expect(universal_1.checkExistsSync('dir', fs, 'a/foo/bar')).to.eql(false);
        });
        it('false for existing file when looking for dir', function () {
            chai_1.expect(universal_1.checkExistsSync('dir', fs, 'foo/bar/a.file')).to.eql(false);
        });
        it('false for non existing dir when looking for file', function () {
            chai_1.expect(universal_1.checkExistsSync('file', fs, 'foo/bar')).to.eql(false);
        });
    });
    describe('checkExists', function () {
        // use NoFeedbackEventsFileSystem as a proxy that does not expose sync methods
        var fs = new no_feedback_events_fs_1.NoFeedbackEventsFileSystem(new universal_1.MemoryFileSystem('', { content: { foo: { bar: { "a.file": 'hello' } } } }));
        it('true for existing file', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = chai_1.expect;
                        return [4 /*yield*/, user_utils_1.checkExists('file', fs, 'foo/bar/a.file')];
                    case 1:
                        _a.apply(void 0, [_b.sent()]).to.eql(true);
                        return [2 /*return*/];
                }
            });
        }); });
        it('true for existing dir', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = chai_1.expect;
                        return [4 /*yield*/, user_utils_1.checkExists('dir', fs, 'foo/bar')];
                    case 1:
                        _a.apply(void 0, [_b.sent()]).to.eql(true);
                        return [2 /*return*/];
                }
            });
        }); });
        it('false for non existing file', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = chai_1.expect;
                        return [4 /*yield*/, user_utils_1.checkExists('file', fs, 'a/foo/bar/a.file')];
                    case 1:
                        _a.apply(void 0, [_b.sent()]).to.eql(false);
                        return [2 /*return*/];
                }
            });
        }); });
        it('false for non existing dir', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = chai_1.expect;
                        return [4 /*yield*/, user_utils_1.checkExists('dir', fs, 'a/foo/bar')];
                    case 1:
                        _a.apply(void 0, [_b.sent()]).to.eql(false);
                        return [2 /*return*/];
                }
            });
        }); });
        it('false for existing file when looking for dir', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = chai_1.expect;
                        return [4 /*yield*/, user_utils_1.checkExists('dir', fs, 'foo/bar/a.file')];
                    case 1:
                        _a.apply(void 0, [_b.sent()]).to.eql(false);
                        return [2 /*return*/];
                }
            });
        }); });
        it('false for non existing dir when looking for file', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = chai_1.expect;
                        return [4 /*yield*/, user_utils_1.checkExists('file', fs, 'foo/bar')];
                    case 1:
                        _a.apply(void 0, [_b.sent()]).to.eql(false);
                        return [2 /*return*/];
                }
            });
        }); });
    });
});
//# sourceMappingURL=user-utils.spec.js.map