"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var chai_1 = require("chai");
var promise_utils_1 = require("../src/promise-utils");
var nodejs_1 = require("../src/nodejs");
var universal_1 = require("../src/universal");
var wamp_client_fs_1 = require("../src/wamp-client-fs");
var implementation_suite_1 = require("./implementation-suite");
var api_1 = require("../src/api");
var sinon_1 = require("sinon");
var msg = 'foo';
var fakeArgs = ['foo', 'bar'];
describe("the wamp client filesystem proxy", function () {
    var wampServer;
    var underlyingFs;
    function server() {
        underlyingFs = new universal_1.MemoryFileSystem();
        return nodejs_1.wampServerOverFs(underlyingFs, 3000);
    }
    function getFS() {
        return Promise.resolve(new universal_1.WampClientFileSystem("ws://127.0.0.1:3000", nodejs_1.wampRealm));
    }
    function getInitedFS() {
        return getFS().then(function (fs) { return fs.init(); });
    }
    var eventMatcherOptions = {
        retries: 25,
        interval: 50,
        noExtraEventsGrace: 150,
        timeout: 1500
    };
    beforeEach(function () { return server().then(function (clientAndServer) { return wampServer = clientAndServer; }); });
    afterEach(function () {
        wampServer.router.close();
        var errMsg = "WAMP connection hasn't been closed after the previous test";
        return promise_utils_1.retryPromise(function () { return wampServer.connection.isConnected ? Promise.reject(errMsg) : Promise.resolve(); }, { interval: 100, retries: 10 });
    });
    api_1.fileSystemAsyncMethods.forEach(function (asyncMethodName) {
        describe(asyncMethodName + " method", function () {
            it("fails when not inited", function () {
                return chai_1.expect(getFS().then(function (fs) { return fs[asyncMethodName].apply(fs, fakeArgs); })).to.eventually.be.rejectedWith(wamp_client_fs_1.noConnectionError);
            });
            it("passes arguments and results correctly", function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                var _this = this;
                var fs, methodImpl, res, _a;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0: return [4 /*yield*/, getInitedFS()];
                        case 1:
                            fs = _b.sent();
                            methodImpl = sinon_1.spy(function () { return tslib_1.__awaiter(_this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
                                return [2 /*return*/, msg];
                            }); }); });
                            underlyingFs[asyncMethodName] = methodImpl;
                            return [4 /*yield*/, fs[asyncMethodName].apply(fs, fakeArgs)];
                        case 2:
                            res = _b.sent();
                            chai_1.expect(res).to.eql(msg);
                            (_a = chai_1.expect(methodImpl).to.have.been).calledWith.apply(_a, fakeArgs);
                            return [2 /*return*/];
                    }
                });
            }); });
            it("reports original error messages", function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                var _this = this;
                var fs;
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, getInitedFS()];
                        case 1:
                            fs = _a.sent();
                            underlyingFs[asyncMethodName] = function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                return tslib_1.__generator(this, function (_a) {
                                    throw new Error(msg);
                                });
                            }); };
                            return [4 /*yield*/, chai_1.expect(fs[asyncMethodName].apply(fs, fakeArgs)).to.eventually.be.rejectedWith(msg)];
                        case 2:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
        });
    });
    implementation_suite_1.assertFileSystemContract(getInitedFS, eventMatcherOptions);
});
//# sourceMappingURL=wamp-client-fs.spec.node.js.map